<?php
require_once __DIR__.'/vendor/autoload.php';

interface IRestrictionManager
{
    public function check();
}

class SimpleRestrictionManager implements IRestrictionManager
{
    const SLEEP = 1;

    const ALLOWED_SPEED = 2;
    const ALLOWED_AMOUNT = 90;

    protected $currentAmount = 0;
    protected $lastDecrement = 0;

    public function check()
    {
        static::decrementStorage();

        if($this->checkStorage())
        {
            echo "!";

            $this->wait();

            return $this->check();
        }

        echo ".";

        $this->incrementStorage();

        return true;
    }

    protected function decrementStorage()
    {
        if($this->lastDecrement > 0)
        {
            $time = time() - $this->lastDecrement;
            $this->currentAmount -= $time * static::ALLOWED_SPEED;

            if($this->currentAmount < 0)
            {
                $this->currentAmount = 0;
            }
        }

        $this->lastDecrement = time();
    }

    protected function checkStorage()
    {
        return $this->currentAmount > static::ALLOWED_AMOUNT;
    }

    protected function incrementStorage()
    {
        $this->currentAmount++;
    }


    protected function wait()
    {
        sleep(static::SLEEP);
    }
}

class Bitrix24WithRestrictions extends \Bitrix24\Bitrix24
{
    /**
     * @var IRestrictionManager
     */
    protected $restrictionManager = null;

    public function setRestrictionManager(IRestrictionManager $manager)
    {
        $this->restrictionManager = $manager;
    }

    public function getRestrictionManager()
    {
        return $this->restrictionManager;
    }

    public function call($methodName, array $additionalParameters = array())
    {
        if($this->getRestrictionManager())
        {
            if(!$this->getRestrictionManager()->check())
            {
                throw new Exception('Restriction failed');
            }
        }

        return parent::call($methodName, $additionalParameters);
    }
}

$app = new Bitrix24WithRestrictions();

$app->setRestrictionManager(new SimpleRestrictionManager());

$app->setDomain('testmodule.bitrix24.ru');
$app->setAccessToken('5jj0zv5xqa114z1kn7m0pcssbw16aaqz');

$amount = 500;
$ts = microtime(true);

echo "Started ".$amount." calls... \n\n";
for($i = 0; $i < $amount; $i++)
{
    //echo $i." ";
    $app->call('app.info', array());
}

$time = (microtime(true) - $ts);

echo "\n\nDone in ".$time.'; speed: '.($amount/$time)." r/s!\n\n";


